import * as discord_js from 'discord.js';
import { GuildTextChannelType, Guild, APIGuildTextChannel, Client, APITextChannel, TextChannel, Message, APIThreadChannel, PublicThreadChannel, ForumChannel, NewsChannel, APIGuildForumChannel, APINewsChannel, TextBasedChannel, User, MessageReaction, GuildChannel, APIInvite, Invite, JSONEncodable, APIEmbed, Embed, APIActionRowComponent, APIMessageActionRowComponent, ActionRowData, MessageActionRowComponentData, MessageActionRowComponentBuilder, ActionRow, MessageActionRowComponent, MessageEditOptions, MessagePayload, InteractionReplyOptions, ClientOptions, APIGuild, PermissionResolvable, Role, GuildBasedChannel, GuildMember, ChatInputCommandInteraction, Interaction, Snowflake, InteractionResponse, ButtonInteraction, APIMessageStringSelectInteractionData, StringSelectMenuInteraction, ClientUser, ClientEvents } from 'discord.js';
import { RawMessageReactionData, RawMessageData, RawRoleData, RawMessageButtonInteractionData, RawMessageComponentInteractionData, RawUserData, RawGuildMemberData } from 'discord.js/typings/rawDataTypes';

declare function getGuildTextChannelMockDataBase<Type extends GuildTextChannelType>(type: Type, guild: Guild): APIGuildTextChannel<Type>;
declare function mockTextChannel(client: Client, guild?: Guild, data?: Partial<APITextChannel>): TextChannel;
declare function mockThreadFromParentMessage(input: {
    client: Client;
    parentMessage: Message;
    data?: Partial<APIThreadChannel>;
}): PublicThreadChannel<boolean>;
declare function mockPublicThread(input: {
    client: Client;
    parentChannel?: TextChannel | ForumChannel | NewsChannel;
    data?: Partial<APIThreadChannel>;
}): PublicThreadChannel<boolean>;
declare function mockForumChannel(client: Client, guild?: Guild, data?: Partial<APIGuildForumChannel>): ForumChannel;
declare function mockNewsChannel(input: {
    client: Client;
    guild?: Guild;
    data?: Partial<APINewsChannel>;
}): NewsChannel;
declare function mockMessages(channel: TextBasedChannel, numberOfMessages: number): Message<boolean>[];
declare function mockMessageReaction({ message, reacter, override, }: {
    message: Message;
    reacter: User;
    override: Partial<RawMessageReactionData>;
}): MessageReaction;
declare function mockReaction({ message, user, override, }: {
    message: Message;
    user: User;
    override?: Partial<RawMessageReactionData>;
}): MessageReaction;
declare function mockMarkedAsSolvedReply({ client, questionId, solutionId, channel, override, }: {
    client: Client;
    questionId: string;
    solutionId: string;
    channel?: TextBasedChannel;
    override?: Partial<RawMessageData>;
}): Message<boolean>;
declare function mockInvite(client: Client, channel: GuildChannel | undefined, override?: Partial<APIInvite>): Invite;

declare function mockEmbed(data: JSONEncodable<APIEmbed> | APIEmbed): Embed;
declare function mockActionRow(data: JSONEncodable<APIActionRowComponent<APIMessageActionRowComponent>> | ActionRowData<MessageActionRowComponentData | MessageActionRowComponentBuilder> | APIActionRowComponent<APIMessageActionRowComponent>): ActionRow<MessageActionRowComponent>;
declare function applyMessagePayload(payload: string | MessageEditOptions | MessagePayload | InteractionReplyOptions, message: Message): Message<boolean>;
declare function mockMessage(input: {
    client: Client;
    author?: User;
    channel?: TextBasedChannel;
    override?: Partial<RawMessageData>;
}): Message<boolean>;

declare function setupBot(override?: Partial<ClientOptions>): Promise<Client<boolean>>;
declare function mockClient(override?: Partial<ClientOptions>): Client<boolean>;
declare function applyClientMocks(client: Client): void;

declare function mockGuild(client: Client, owner?: User, data?: Partial<APIGuild>): Guild;
declare function mockRole(client: Client, permissions: PermissionResolvable, guild?: Guild, role?: Partial<RawRoleData>): Role;

declare function mockChatInputCommandInteraction({ client, name, id, channel, member, }: {
    client: Client;
    name: string;
    id: string;
    channel?: GuildBasedChannel;
    member?: GuildMember;
}): ChatInputCommandInteraction;
declare function mockInteractionResponse({ interaction, id, }: {
    interaction: Interaction;
    id: Snowflake;
}): InteractionResponse;
declare function mockButtonInteraction({ override, caller, message, }: {
    caller: User;
    message: Message;
    override?: Partial<Omit<RawMessageButtonInteractionData & RawMessageComponentInteractionData, "component_type">>;
}): ButtonInteraction<discord_js.CacheType>;
declare function mockStringSelectInteraction({ override, caller, message, data, }: {
    caller: User;
    message: Message;
    data: Omit<APIMessageStringSelectInteractionData, "component_type" | "values"> & {
        values: string[] | string;
    };
    override?: Partial<Omit<RawMessageComponentInteractionData, "data">>;
}): StringSelectMenuInteraction<discord_js.CacheType>;

declare function mockUser(client: Client, data?: Partial<RawUserData>): User;
declare function mockClientUser(client: Client, override?: Partial<RawUserData>): ClientUser;
declare function mockGuildMember(input: {
    client: Client;
    user?: User;
    guild?: Guild;
    permissions?: PermissionResolvable;
    data?: Partial<RawGuildMemberData>;
}): GuildMember;

declare function delay(timeInMs?: number): Promise<void>;
declare function emitEvent<E extends keyof ClientEvents>(client: Client, event: E, ...args: ClientEvents[E]): Promise<boolean>;
declare function overrideVariables<T extends {}>(obj: T, overrides: {}): void;
declare function copyClass<T extends {
    client: Client;
}>(obj: T, client: Client, overrides?: {}): T;
type PermissionVariantsTest = {
    permissionsThatShouldWork: PermissionResolvable[];
    operation: (permission: PermissionResolvable, isPermissionAllowed: boolean) => Promise<void> | void;
};
declare function testAllPermissions({ permissionsThatShouldWork, operation, }: PermissionVariantsTest): Promise<void>;

type GuildMemberVariants = Awaited<ReturnType<typeof createGuildMemberVariants>>;
declare function createGuildMemberVariants(client: Client, guild?: Guild | undefined): Promise<{
    guildMemberOwner: discord_js.GuildMember;
    guildMemberDefault: discord_js.GuildMember;
    pendingGuildMemberDefault: discord_js.GuildMember;
    guildMemberManageGuild: discord_js.GuildMember;
    guildMemberAdmin: discord_js.GuildMember;
}>;

declare global {
    namespace NodeJS {
        interface ProcessEnv {
            DEFAULT_DELAY_IN_MS: string | undefined;
        }
    }
}

export { GuildMemberVariants, PermissionVariantsTest, applyClientMocks, applyMessagePayload, copyClass, createGuildMemberVariants, delay, emitEvent, getGuildTextChannelMockDataBase, mockActionRow, mockButtonInteraction, mockChatInputCommandInteraction, mockClient, mockClientUser, mockEmbed, mockForumChannel, mockGuild, mockGuildMember, mockInteractionResponse, mockInvite, mockMarkedAsSolvedReply, mockMessage, mockMessageReaction, mockMessages, mockNewsChannel, mockPublicThread, mockReaction, mockRole, mockStringSelectInteraction, mockTextChannel, mockThreadFromParentMessage, mockUser, overrideVariables, setupBot, testAllPermissions };
